@@ -11,13 +11,10 @@
 #include <petsc/private/tsimpl.h>
 #include <petsc/private/taoimpl.h>
 #include <petsc/private/deviceimpl.h>
+#include <petsc/private/viewerimpl.h>
 
 /* ---------------------------------------------------------------- */
 
-#ifndef PETSC_ERR_PYTHON
-#define PETSC_ERR_PYTHON ((PetscErrorCode)(-1))
-#endif
-
 #define PetscERROR(comm,FUNCT,n,t,msg,arg) \
         PetscError(comm,__LINE__,FUNCT,__FILE__,n,t,msg,arg)
 
@@ -231,31 +228,6 @@ PetscErrorCode MatHasPreallocationAIJ(Mat A,PetscBool *aij,PetscBool *baij,Petsc
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-static
-PetscErrorCode MatGetCurrentMemType(Mat A, PetscMemType *m)
-{
-  PetscBool bound;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(A,MAT_CLASSID,1);
-  PetscAssertPointer(m,2);
-  *m = PETSC_MEMTYPE_HOST;
-  PetscCall(MatBoundToCPU(A,&bound));
-  if (!bound) {
-    VecType rtype;
-    char *iscuda = NULL, *iship = NULL, *iskok = NULL;
-
-    PetscCall(MatGetRootType_Private(A,&rtype));
-    PetscCall(PetscStrstr(rtype,"cuda",&iscuda));
-    PetscCall(PetscStrstr(rtype,"hip",&iship));
-    PetscCall(PetscStrstr(rtype,"kokkos",&iskok));
-    if (iscuda)     *m = PETSC_MEMTYPE_CUDA;
-    else if (iship) *m = PETSC_MEMTYPE_HIP;
-    else if (iskok) *m = PETSC_MEMTYPE_KOKKOS;
-  }
-  PetscFunctionReturn(PETSC_SUCCESS);
-}
-
 #ifndef MatNullSpaceFunction
 typedef PetscErrorCode MatNullSpaceFunction(MatNullSpace,Vec,void*);
 #endif
@@ -364,9 +336,9 @@ PetscErrorCode KSPConverged(KSP ksp,PetscInt iter,PetscReal rnorm,KSPConvergedRe
 
 typedef struct {
   PetscBool prepend_custom;
-  PetscErrorCode (*convtest)(KSP, PetscInt, PetscReal, KSPConvergedReason *, void *);
-  PetscErrorCode (*convdestroy)(void *);
-  PetscErrorCode (*convtestcustom)(KSP, PetscInt, PetscReal, KSPConvergedReason *, void *);
+  KSPConvergenceTestFn *convtest;
+  PetscCtxDestroyFn    *convdestroy;
+  KSPConvergenceTestFn *convtestcustom;
   void *convctx;
 } KSPConvergedNativeCtx;
 
@@ -396,13 +368,13 @@ PetscErrorCode KSPConvergedNative_Private(KSP ksp, PetscInt n, PetscReal rnorm,
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-static PetscErrorCode KSPConvergedNative_Destroy(void *cctx)
+static PetscErrorCode KSPConvergedNative_Destroy(void **cctx)
 {
-  KSPConvergedNativeCtx *ctx = (KSPConvergedNativeCtx *)cctx;
+  KSPConvergedNativeCtx *ctx = (KSPConvergedNativeCtx *)*cctx;
 
   PetscFunctionBegin;
   PetscCheck(ctx, PETSC_COMM_SELF, PETSC_ERR_PLIB, "Missing context");
-  if (ctx->convdestroy) PetscCall((*ctx->convdestroy)(ctx->convctx));
+  if (ctx->convdestroy) PetscCall((*ctx->convdestroy)(&ctx->convctx));
   PetscCall(PetscFree(ctx));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
